#ifndef TABLE_SYM_H

#define TABLE_SYM_H

#include <stdio.h>
#include "outils.h"

#define max_var 1000 /* le nombre maximal de variables possibles */
#define max_fun 100 /* le nombre maximal de fonctions possibles */
#define max_tab 50 /* le nombre maximal de tableaux possibles */
#define max_param 20 /* le nombre maximal de parametres possibles */
#define max_boucle 100 /* nombre maximal de boucle */
#define max_sort 30 /* definie le nombre maximal de sortir dans une boucle */
#define max_hyper 10 /* definie le nombre maximal d'hyperboucle */
#define max_enfonce 30 /* le nombre de enfonce maximal d'une hyperboucle */

/* Une variable a un nom et une adresse */
typedef struct Cellule_var
 {
  char nom_var[max_car];
  int adresse;
 } cellule_var;


/* Un tableau a un nom,une adresse de depart et une taille.
L'adresse de depart contient  l'adresse du dbut des donnes
 et l'adresse suivante la taille du tablo (c'est pour le systeme) */
typedef struct Cellule_tab
 {
  char nom_tab[max_car];
  int adresse;
  int taille;
 } cellule_tab;

/* Idem, mais on met un type */
typedef struct Cellule_param
 {
  char nom_param[max_car];
  int adresse;
  bool type; /* 0 var , 1 tablo */
 } cellule_param;


/* *************** La table des variables globales ***************** */
cellule_var table_var_global[max_var];
int nb_var_global;

/* *************** La table des tableaux globaux ******************* */
cellule_tab table_tab_global[max_tab];
int nb_tab_global;

typedef struct Cellule_fun
 {
  char nom_fun[max_car]; /* Nom de la fonction */
  cellule_var table_var[max_var]; /* Table des variables de la fonction */
  cellule_tab table_tab[max_tab]; /* Table des tableaux de la fonction */
  cellule_param table_param[max_param]; /* Tables des parametres */
  int nb_param; /* nombre de parametre */
  int nb_var; /* nombre de variables */
  int nb_tab; /* nombre de tablaux */
  int adresse; /* adresse de dbut de code */
 } cellule_fun;

/* ********************* La table des symboles ************************** */
cellule_fun table[max_fun];

/* *********** nombre de fonctions lu ************** */
int nb_fun_lu;

/* ****************** La table des parametres pour la fonction ************ */
int nb_parametre;
cellule_param parametre_en_cour[max_param];

/* ************** Taille quand on crer un tableau *********** */
int taille;


/* ****************** Chaque boucle a sa structure ********************** */
typedef struct Cellule_boucle
 {
  int adr_jz; /* adresse de debut de boucl, le JZ voir rapport */
  int val_jz; /* adresse du compteur de boucle */
  int adr_jmp; /* adresse du JMP al a fin de la boucle */
  int nb_sort; /* nombre de "sortir" crois dans la boucle */
  int sort[max_sort]; /* table des "sortir" */
 } cellule_boucle;

/* *********************** Table des boucle ****************** */
cellule_boucle pile_boucle[max_boucle];
int nb_boucle; /* nombre de boucle */ 


/* ******************* Chaque Hyperboucle a sa structure ***************** */
typedef struct Cellule_hyper
 {
  int adr_test;  /* test si il faut entrer ou non dans l'hyperboucle */
  int adr_debut; /* Adresse de debut de code*/
  int adr_fin;   /* Adresse de fin de code*/
  int adr_final; /* Adresse du code du final */
  int adr_compt; /* adresse du compteur */
  int adr_max; /* adresse du compteur de maximal voir rapport */
  int adr_tmp; /* adresse de la lecture tempporaire */
  int adr_je;  /* adresse du JE de fin de hyperboucle voir rapport */
  int enfonce[max_enfonce]; /* la table des "enfonce" lu */
  int nb_enfonce; /* le onmbre de "enfoncer" lu */
  int adr_env[max_var]; /* le variables temporaire a sauver */
  int nb_tmp;          /* le nombrede celle-ci */
  int nb_push_hyper;   /* le nombre de pusher des temporaire a faire */
 } cellule_hyper;

/* ********************** Table des hyperboucles ***************************** */
cellule_hyper pile_hyper[max_hyper];
int nb_hyper; /* nombrede hyperboucle */


void init_table(void);/* initialisation de la table des symboles */

int cherche_adresse_var(char *name_var,char *name_fun);/* Recherche adresse d'une variable */
int creation_var(char *name_var,char *name_fun); /* Fonction de cration d'une variable */
int creation_var_global(char *name_var); /* Fonction de cration d'une variable global */
int parametre_var(char *name_var,char *name_fun,int type);/* Cration d'un parametre d'une fonctionlors des dfinitions */

int cherche_adresse_tab(char *name_tab,char *name_fun);/* Recherche adresse d'un tableaux */
int creation_tab(char *name_tab,char *name_fun,int taille); /* Fonction de cration d'un tableau */
int creation_tab_global(char *name_tab,int taille); /* Fonction de cration d'un tableau global */

void change_fun(void); /* Changer de fonction */
void mettre_programme(void); /* Mettre le programme principal dans la table des symboles */

int ajoute_en_param(char *name,char *nom_fun,int type); /* ajoute un objet en parametre */

#endif
